@@ -107,7 +107,7 @@ def compile_path(
         ), f"Unknown path convertor '{convertor_type}'"
         convertor = CONVERTOR_TYPES[convertor_type]
 
-        path_regex += path[idx : match.start()]
+        path_regex += re.escape(path[idx : match.start()])
         path_regex += f"(?P<{param_name}>{convertor.regex})"
 
         path_format += path[idx : match.start()]
@@ -117,7 +117,7 @@ def compile_path(
 
         idx = match.end()
 
-    path_regex += path[idx:] + "$"
+    path_regex += re.escape(path[idx:]) + "$"
     path_format += path[idx:]
 
     return re.compile(path_regex), path_format, param_convertors
@@ -455,6 +455,7 @@ class Router:
         default: ASGIApp = None,
         on_startup: typing.Sequence[typing.Callable] = None,
         on_shutdown: typing.Sequence[typing.Callable] = None,
+        lifespan: typing.Callable[[typing.Any], typing.AsyncGenerator] = None,
     ) -> None:
         self.routes = [] if routes is None else list(routes)
         self.redirect_slashes = redirect_slashes
@@ -462,6 +463,13 @@ class Router:
         self.on_startup = [] if on_startup is None else list(on_startup)
         self.on_shutdown = [] if on_shutdown is None else list(on_shutdown)
 
+        async def default_lifespan(app: typing.Any) -> typing.AsyncGenerator:
+            await self.startup()
+            yield
+            await self.shutdown()
+
+        self.lifespan_context = default_lifespan if lifespan is None else lifespan
+
     async def not_found(self, scope: Scope, receive: Receive, send: Send) -> None:
         if scope["type"] == "websocket":
             websocket_close = WebSocketClose()
@@ -510,21 +518,29 @@ class Router:
         Handle ASGI lifespan messages, which allows us to manage application
         startup and shutdown events.
         """
+        first = True
+        app = scope.get("app")
         message = await receive()
-        assert message["type"] == "lifespan.startup"
-
         try:
-            await self.startup()
+            if inspect.isasyncgenfunction(self.lifespan_context):
+                async for item in self.lifespan_context(app):
+                    assert first, "Lifespan context yielded multiple times."
+                    first = False
+                    await send({"type": "lifespan.startup.complete"})
+                    message = await receive()
+            else:
+                for item in self.lifespan_context(app):  # type: ignore
+                    assert first, "Lifespan context yielded multiple times."
+                    first = False
+                    await send({"type": "lifespan.startup.complete"})
+                    message = await receive()
         except BaseException:
-            msg = traceback.format_exc()
-            await send({"type": "lifespan.startup.failed", "message": msg})
+            if first:
+                exc_text = traceback.format_exc()
+                await send({"type": "lifespan.startup.failed", "message": exc_text})
             raise
-
-        await send({"type": "lifespan.startup.complete"})
-        message = await receive()
-        assert message["type"] == "lifespan.shutdown"
-        await self.shutdown()
-        await send({"type": "lifespan.shutdown.complete"})
+        else:
+            await send({"type": "lifespan.shutdown.complete"})
 
     async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
         """
@@ -556,7 +572,7 @@ class Router:
         if partial is not None:
             #  Handle partial matches. These are cases where an endpoint is
             # able to handle the request, but is not a preferred option.
-            # We use this in particular to deal with "406 Method Not Found".
+            # We use this in particular to deal with "405 Method Not Allowed".
             scope.update(partial_scope)
             await partial.handle(scope, receive, send)
             return
